/*
 * Bookr: document reader for the Sony PSP
 * Copyright (C) 2005 Carlos Carrasco Martinez (carloscm at gmail dot com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <string.h>
#include <vector>
#include "fzscreen.h"

#include "bkmainmenu.h"
#include "bkdocument.h"
#include "bkbookmark.h"
#include "bkpopup.h"
#include "bkuser.h"
#include "bksql.h"

BKMainMenu* BKMainMenu::create() {
	BKMainMenu* f = new BKMainMenu();
	FZScreen::resetReps();
	return f;
}

void BKMainMenu::setMode(int newMode) {
	mode = newMode;
}

BKMainMenu::BKMainMenu() : mode(BKMM_MAIN), captureButton(false), frames(0) {
	BKMenuItem mi = BKMenuItem("Open Book", "Select", BK_MENU_ITEM_OPTIONAL_TRIANGLE_LABEL);
	mi.triangleLabel = "Last Open";
	mainItems.push_back(mi);
//	mainItems.push_back(BKMenuItem("Open file", "Select", 0));
	mainItems.push_back(BKMenuItem("Controls", "Select", 0));	
	mainItems.push_back(BKMenuItem("Options", "Select", 0));
	mainItems.push_back(BKMenuItem("About", "Select", 0));
	mainItems.push_back(BKMenuItem("Exit", "Select", 0));
	buildControlMenu();
	buildOptionMenu();
	buildBookMenu();
	buildSearchMenu();
	buildResultsMenu();
}

BKMainMenu::~BKMainMenu() {
}

void BKMainMenu::rebuildMenu() {
	if (mode == BKMM_CONTROLS) {
		buildControlMenu();
	} else if (mode == BKMM_OPTIONS) {
		buildOptionMenu();
	} else if (mode == BKMM_BOOKS) {
		buildBookMenu();
	} else if (mode == BKMM_SEARCH) {
		buildSearchMenu();
	}
}

int BKMainMenu::update(unsigned int buttons) {
	if (mode == BKMM_CONTROLS) {
		return updateControls(buttons);
	} else if (mode == BKMM_OPTIONS) {
		return updateOptions(buttons);
	} else if (mode == BKMM_BOOKS) {
		return updateBooks(buttons);
	} else if (mode == BKMM_SEARCH) {
		return updateSearch(buttons);
	} else if (mode == BKMM_RESULTS) {
		// if there's only one result then open it and skip menu
		if( BKSql::searchResults.size() == 1 ){
			BKSql::searchResult = 0;
			return BK_CMD_LOAD_SEARCH;
		}
		return updateResults(buttons);
	}
	return updateMain(buttons);
}

void BKMainMenu::render() {
	string t("");
	if (mode == BKMM_MAIN) {
		string title("Main Menu");
		drawMenu(title, t, mainItems);
		string e;
		drawClockAndBattery(e);
	} else if (mode == BKMM_CONTROLS) {
		string title("Customize controls");
		drawMenu(title, t, controlItems);
	} else if (mode == BKMM_OPTIONS) {
		string title("Options");
		drawMenu(title, t, optionItems);
	} else if (mode == BKMM_BOOKS) {
		string title("Load Book");
		drawMenu(booksText, t, bookItems);
	} else if (mode == BKMM_SEARCH) {
		string title("Search Menu");
		drawMenu(title, t, searchItems);
	} else if (mode == BKMM_RESULTS) {
		if( BKSql::searchResults.size() == 1 ){ return; }
		string title("Search Results");
		drawMenu(title, t, resultsItems);
	}
	if (captureButton) {
		texUI->bindForDisplay();
		FZScreen::ambientColor(0xf0dddddd|0xf00000ff);
		drawPill(150, 126, 180, 20, 6, 31, 1);
		fontBig->bindForDisplay();
		FZScreen::ambientColor(0xff222222|0xf00000ff);
		drawTextHC("Press the new button", fontBig, 130);
	}
}

int BKMainMenu::updateMain(unsigned int buttons) {
	menuCursorUpdate(buttons, mainItems);

	int* b = FZScreen::ctrlReps();

	if (b[(BKUser::options.btnSwap?FZ_REPS_CROSS:FZ_REPS_CIRCLE)] == 1) {

		if (selItem == 0) {
			selItem = 0;
			topItem = 0;
			mode = BKMM_BOOKS;
			return BK_CMD_MARK_DIRTY;
		}
//		if (selItem == 1) {
//			return BK_CMD_INVOKE_OPEN_FILE;
//		}
		if (selItem == 1) {
			selItem = 0;
			topItem = 0;
			mode = BKMM_CONTROLS;
			return BK_CMD_MARK_DIRTY;
		}
		if (selItem == 2) {
			selItem = 0;
			topItem = 0;
			mode = BKMM_OPTIONS;
			return BK_CMD_MARK_DIRTY;
		}
		if (selItem == 3) {
			popupText = "The Good Bookr 1.1 - a Bible reader for the Sony PSP."
						"\nBased on Bookr http://bookr.sf.net/"
						"\nThis program is licensed under the terms of the GPL v2.";
			popupMode = BKPOPUP_INFO;
			return BK_CMD_MAINMENU_POPUP;
		}
		if (selItem == 4) {
			return BK_CMD_EXIT;
		}
	}

	if (b[FZ_REPS_TRIANGLE] == 1) {
		if (selItem == 0) {
			BKBookmark b;
			string bm = "book";
			if (BKBookmarksManager::getLastView(bm, b)) {
//				doc->setBookmarkPosition(b.viewData);
				BKUser::options.lastBook = b.book;
				BKUser::options.lastChapter = b.chapter;
			}
			return BK_CMD_LOAD_BOOK;
		}
	}

	if (b[(BKUser::options.btnSwap?FZ_REPS_CIRCLE:FZ_REPS_CROSS)] == 1) {
		return BK_CMD_CLOSE_TOP_LAYER;
	}

	if (b[FZ_REPS_START] == 1) {
		return BK_CMD_CLOSE_TOP_LAYER;
	}

	frames++;
	if (frames % 60 == 0)
		return BK_CMD_MARK_DIRTY;

	return 0;
}

/*** POPUPS ***/

string BKMainMenu::getPopupText() {
	return popupText;
}

int BKMainMenu::getPopupMode() {
	return popupMode;
}

/*** SEARCH MENU ***/

void BKMainMenu::buildSearchMenu() {

	searchItems.clear();
	BKMenuItem mi;
	searchCt = BKSql::searchResults.empty() ? 0 : BKSql::searchResults.size();			

	bool more = searchCt > 1 && BKSql::searchResult < searchCt-1;

	mi = BKMenuItem("New Search", "Select", 0);
	if( more ){
		mi.flags = BK_MENU_ITEM_OPTIONAL_TRIANGLE_LABEL;
		mi.triangleLabel = "Next Result";
	}
	searchItems.push_back(mi);

	mi = BKMenuItem("Next Result: ", "Select", 0);
	if( more ){
		mi.label.append(BKSql::searchResults[BKSql::searchResult+1].name);
	} else mi.disabled = true;
	searchItems.push_back(mi);

	mi = BKMenuItem("Previous Result: ", "Select", 0);
	if( searchCt > 1 && BKSql::searchResult != 0 ){
		mi.label.append(BKSql::searchResults[BKSql::searchResult-1].name);
	} else mi.disabled = true;
	searchItems.push_back(mi);

	mi = BKMenuItem("All Results", "Select", 0);
	if( BKSql::searchResults.empty() ){
		mi.disabled = true;
	} else {
		char label[500];
		if( searchCt == 1 ){
			sprintf(label, "Open %s", BKSql::searchResults[0].name.c_str());
		} else {
			sprintf(label, "All %i Results for \"%s\"", searchCt, BKSql::searchTerm.c_str());
		}
		mi.label = label;
	}
	searchItems.push_back(mi);

	mi = BKMenuItem("Search Within: ", "Change", BK_MENU_ITEM_USE_LR_ICON | BK_MENU_ITEM_OPTIONAL_TRIANGLE_LABEL);
	mi.label.append(BKSql::searchWithins[BKSql::searchWithin].label);
	mi.triangleLabel = "New Search";
	if( BKSql::searchMatches[BKSql::searchMatch].key == SEARCH_MATCH_VERSE ){
		mi.disabled = true;
	}
	searchItems.push_back(mi);

	mi = BKMenuItem("Search Matching: ", "Change", BK_MENU_ITEM_USE_LR_ICON | BK_MENU_ITEM_OPTIONAL_TRIANGLE_LABEL);
	mi.label.append(BKSql::searchMatches[BKSql::searchMatch].label);
	mi.triangleLabel = "New Search";
	searchItems.push_back(mi);

}

int BKMainMenu::updateSearch(unsigned int buttons) {
	menuCursorUpdate(buttons, searchItems);

	int* b = FZScreen::ctrlReps();

	bool more = searchCt > 1 && BKSql::searchResult < searchCt-1;

	if (b[(BKUser::options.btnSwap?FZ_REPS_CROSS:FZ_REPS_CIRCLE)] == 1) {
		switch(selItem){
			case 0: return BK_CMD_OSK; break;
			case 1: BKSql::searchResult++; return BK_CMD_LOAD_SEARCH; break;
			case 2: BKSql::searchResult--; return BK_CMD_LOAD_SEARCH; break;
			case 3: return BK_CMD_SEARCH_RESULTS_ALL; break;
		}
	}
	if( b[FZ_REPS_TRIANGLE] == 1 ){
		switch(selItem){
			case 0: if(more){ BKSql::searchResult++; return BK_CMD_LOAD_SEARCH; } break;
			case 4: case 5: return BK_CMD_OSK; break;
		}
	}

	if ( b[FZ_REPS_LEFT] == 1 ) {
		if ( selItem == 4 ) {
			if(BKSql::searchWithin) BKSql::searchWithin--;
			else BKSql::searchWithin = BKSql::searchWithins.size()-1;
			buildSearchMenu();
		} else if ( selItem == 5 ) {
			if(BKSql::searchMatch) BKSql::searchMatch--;
			else BKSql::searchMatch = BKSql::searchMatches.size()-1;
			buildSearchMenu();
		}
		return BK_CMD_MARK_DIRTY;
	}
	if ( b[FZ_REPS_RIGHT] == 1 ) {
		if ( selItem == 4 ) {
			if(BKSql::searchWithin < BKSql::searchWithins.size()-1) BKSql::searchWithin++;
			else BKSql::searchWithin = 0;
			buildSearchMenu();
		} else if ( selItem == 5 ) {
			if(BKSql::searchMatch < BKSql::searchMatches.size()-1) BKSql::searchMatch++;
			else BKSql::searchMatch = 0;
			buildSearchMenu();
		}
		return BK_CMD_MARK_DIRTY;
	}
	
	if (b[FZ_REPS_START] == 1 || b[(BKUser::options.btnSwap?FZ_REPS_CIRCLE:FZ_REPS_CROSS)] == 1) {
		return BK_CMD_CLOSE_TOP_LAYER;
	}

	return 0;
}

/*** SEARCH RESULTS ***/

void BKMainMenu::buildResultsMenu() {

	resultsItems.clear();
	if( !BKSql::searchResults.empty() ){
		for( unsigned int i = 0; i < BKSql::searchResults.size(); i++ ) {
			resultsItems.push_back( BKMenuItem(BKSql::searchResults[i].name, "Load", 0) );
		}
	}

}

int BKMainMenu::updateResults(unsigned int buttons) {

	menuCursorUpdate(buttons, resultsItems);

	int* b = FZScreen::ctrlReps();

	if (b[(BKUser::options.btnSwap?FZ_REPS_CROSS:FZ_REPS_CIRCLE)] == 1) {
		BKSql::searchResult = selItem;
		return BK_CMD_LOAD_SEARCH;
	}

	if (b[FZ_REPS_START] == 1 || b[(BKUser::options.btnSwap?FZ_REPS_CIRCLE:FZ_REPS_CROSS)] == 1) {
		return BK_CMD_CLOSE_TOP_LAYER;
	}

	return 0;
}

/*** LOAD BOOK ***/

void BKMainMenu::buildBookMenu() {

	bookItems.clear();

	for( unsigned int i = 0; i < BKSql::books.size(); i++ ) {
		bookItems.push_back( BKMenuItem(BKSql::books[i].name, "Load", 0) );
	}

}

int BKMainMenu::updateBooks(unsigned int buttons) {
	menuCursorUpdate(buttons, bookItems);

	int* b = FZScreen::ctrlReps();

	// label OT/NT
	if( selItem < 39 )
		booksText = "Old Testament";
	else
		booksText = "New Testament";

	// label book type
	if( selItem < 5 )
		booksText.append( ": Pentateuch/Torah - Books of Moses" );
	else if( selItem < 17 )
		booksText.append( ": History" );
	else if( selItem < 22 )
		booksText.append( ": Poetry" );
	else if( selItem < 27 )
		booksText.append( ": Major Prophets" );
	else if( selItem < 39 )
		booksText.append( ": Minor Prophets" );
	else if( selItem < 43 )
		booksText.append( ": Gospels" );
	else if( selItem < 44 )
		booksText.append( ": History" );
	else if( selItem < 58 )
		booksText.append( ": Pauline Epistles - Letters of Paul" );
	else if( selItem < 65 )
		booksText.append( ": General Letters" );
	else if( selItem < 66 )
		booksText.append( ": Prophecy" );

	if (b[(BKUser::options.btnSwap?FZ_REPS_CROSS:FZ_REPS_CIRCLE)] == 1) {
		BKUser::options.lastBook = selItem+1;
		BKUser::options.lastChapter = 1;
		return BK_CMD_LOAD_BOOK;
	}

	if (b[(BKUser::options.btnSwap?FZ_REPS_CIRCLE:FZ_REPS_CROSS)] == 1) {
		selItem = 0;
		topItem = 0;
		mode = BKMM_MAIN;
		return BK_CMD_MARK_DIRTY;
	}

	if (b[FZ_REPS_START] == 1) {
		return BK_CMD_CLOSE_TOP_LAYER;
	}

	return 0;
}

/*** CONTROLS ***/

void BKMainMenu::buildControlMenu() {
	string cb("Change button");

	controlItems.clear();

	string t("Search Menu: ");
	t += FZScreen::nameForButtonReps(BKUser::controls.searchMenu);
	controlItems.push_back(BKMenuItem(t, cb, 0));

	t = "Previous Chapter: ";
	t += FZScreen::nameForButtonReps(BKUser::controls.previousChapter);
	controlItems.push_back(BKMenuItem(t, cb, 0));

	t = "Next Chapter: ";
	t += FZScreen::nameForButtonReps(BKUser::controls.nextChapter);
	controlItems.push_back(BKMenuItem(t, cb, 0));

	t = "Previous Book: ";
	t += FZScreen::nameForButtonReps(BKUser::controls.previousBook);
	controlItems.push_back(BKMenuItem(t, cb, 0));

	t = "Next Book: ";
	t += FZScreen::nameForButtonReps(BKUser::controls.nextBook);
	controlItems.push_back(BKMenuItem(t, cb, 0));

	t = "Scroll Up: ";
	t += FZScreen::nameForButtonReps(BKUser::controls.screenUp);
	controlItems.push_back(BKMenuItem(t, cb, 0));

	t = "Scroll Down: ";
	t += FZScreen::nameForButtonReps(BKUser::controls.screenDown);
	controlItems.push_back(BKMenuItem(t, cb, 0));

	t = "Previous Page: ";
	t += FZScreen::nameForButtonReps(BKUser::controls.screenLeft);
	controlItems.push_back(BKMenuItem(t, cb, 0));

	t = "Next Page: ";
	t += FZScreen::nameForButtonReps(BKUser::controls.screenRight);
	controlItems.push_back(BKMenuItem(t, cb, 0));

	controlItems.push_back(BKMenuItem("Restore defaults", "Select", 0));

}

static int buttonsHack = 0;
int BKMainMenu::updateControls(unsigned int buttons) {
	const int validMask = FZ_CTRL_UP | FZ_CTRL_RIGHT | FZ_CTRL_DOWN | FZ_CTRL_LEFT | FZ_CTRL_LTRIGGER |
		FZ_CTRL_RTRIGGER | FZ_CTRL_TRIANGLE | FZ_CTRL_CIRCLE | FZ_CTRL_CROSS | FZ_CTRL_SQUARE;
	if (captureButton) {
		// not pretty... wait for the mask to become 0 at least once
		if (buttons != 0 && buttonsHack == 0)
			return 0;
		buttonsHack = 1;
		// capture only non-system buttons
		if ((buttons & validMask) == 0)
			return 0;
		int repsCode = FZScreen::repsForButtonMask(buttons);
		switch (selItem) {
			case  0: BKUser::controls.searchMenu = repsCode; break;
			case  1: BKUser::controls.previousChapter = repsCode; break;
			case  2: BKUser::controls.nextChapter = repsCode; break;
			case  3: BKUser::controls.previousBook = repsCode; break;
			case  4: BKUser::controls.nextBook = repsCode; break;
			case  5: BKUser::controls.screenUp = repsCode; break;
			case  6: BKUser::controls.screenDown = repsCode; break;
			case  7: BKUser::controls.screenLeft = repsCode; break;
			case  8: BKUser::controls.screenRight = repsCode; break;
		}
		BKUser::save();
		buildControlMenu();
		captureButton = false;
		return BK_CMD_MARK_DIRTY;
	}

	menuCursorUpdate(buttons, controlItems);

	int* b = FZScreen::ctrlReps();

	if (b[(BKUser::options.btnSwap?FZ_REPS_CROSS:FZ_REPS_CIRCLE)] == 1) {
		if (selItem == 9) {
			BKUser::setDefaultControls();
			BKUser::save();
			buildControlMenu();			
			return BK_CMD_MARK_DIRTY;
		}
		buttonsHack = 0;
		captureButton = true;
		return BK_CMD_MARK_DIRTY;
	}

	if (b[(BKUser::options.btnSwap?FZ_REPS_CIRCLE:FZ_REPS_CROSS)] == 1) {
		selItem = 0;
		topItem = 0;
		mode = BKMM_MAIN;
		return BK_CMD_MARK_DIRTY;
	}

	if (b[FZ_REPS_START] == 1) {
		return BK_CMD_CLOSE_TOP_LAYER;
	}

	return 0;
}

/*** OPTIONS ***/

void BKMainMenu::buildOptionMenu() {

	optionItems.clear();

	string t("");

	t = "Font: ";
	if (BKUser::options.txtFont == "bookr:builtin") {
		t += "built-in";
	} else {
		int s = BKUser::options.txtFont.size();
		if (s > 25) {
			t += "...";
			t += string(BKUser::options.txtFont, s - 25, 25);
		} else {
			t += BKUser::options.txtFont;
		}
	}
	BKMenuItem mi = BKMenuItem(t, "Select font", BK_MENU_ITEM_OPTIONAL_TRIANGLE_LABEL);
	mi.triangleLabel = "Use built-in font";
	optionItems.push_back(mi);

	char txt[1024];
	snprintf(txt, 1024, "Font size: %d pixels", BKUser::options.txtSize);
	t = txt;
	optionItems.push_back(BKMenuItem(t, "Change", BK_MENU_ITEM_USE_LR_ICON));

	t = "Justify text: ";
	t += BKUser::options.txtJustify ? "Enabled" : "Disabled";
	optionItems.push_back(BKMenuItem(t, "Toggle", 0));

	t = "Text Foreground color: ";
	mi = BKMenuItem(t, "Select", BK_MENU_ITEM_COLOR_RECT);
	mi.color = BKUser::options.txtFGColor;
	optionItems.push_back(mi);

	t = "Text Background color: ";
	mi = BKMenuItem(t, "Select", BK_MENU_ITEM_COLOR_RECT);
	mi.color = BKUser::options.txtBGColor;
	optionItems.push_back(mi);

	snprintf(txt, 1024, "CPU/Bus speed: %s", FZScreen::speedLabels[BKUser::options.pspSpeed]);
	t = txt;
	optionItems.push_back(BKMenuItem(t, "Change", BK_MENU_ITEM_USE_LR_ICON));

	t = "Display page loading and numbering labels: ";
	t += BKUser::options.displayLabels ? "Enabled" : "Disabled";
	optionItems.push_back(BKMenuItem(t, "Toggle", 0));

	optionItems.push_back(BKMenuItem("Clear bookmarks", "Select", 0));

	t = "Buttons: ";
	t += BKUser::options.btnSwap ? "Cross Selects, Circle Closes" : "Circle Selects, Cross Closes";
	optionItems.push_back(BKMenuItem(t, "Change", BK_MENU_ITEM_USE_LR_ICON));

	optionItems.push_back(BKMenuItem("Restore defaults", "Select", 0));

}

int BKMainMenu::updateOptions(unsigned int buttons) {
	menuCursorUpdate(buttons, optionItems);

	int* b = FZScreen::ctrlReps();

	if (b[(BKUser::options.btnSwap?FZ_REPS_CROSS:FZ_REPS_CIRCLE)] == 1) {
		if (selItem == 0) {
			return BK_CMD_INVOKE_OPEN_FONT;
		}
		// 1 = font size
		if (selItem == 2) {
			BKUser::options.txtJustify = !BKUser::options.txtJustify;
			buildOptionMenu();
			return BK_CMD_MARK_DIRTY;
		}
		if (selItem == 3) {
			return BK_CMD_INVOKE_COLOR_CHOOSER_TXTFG;
		}
		if (selItem == 4) {
			return BK_CMD_INVOKE_COLOR_CHOOSER_TXTBG;
		}
		// 5 = CPU speed
		if (selItem == 6) {
			BKUser::options.displayLabels = !BKUser::options.displayLabels;
			buildOptionMenu();
			return BK_CMD_MARK_DIRTY;
		}
		if (selItem == 7) {
			BKBookmarksManager::clear();
			popupText = "Bookmarks cleared.";
			popupMode = BKPOPUP_INFO;
			return BK_CMD_MAINMENU_POPUP;
		}
		// 8 = Buttons
		if (selItem == 9) {
			BKUser::setDefaultOptions();
			buildOptionMenu();
			return BK_CMD_MARK_DIRTY;
		}
	}

	if (b[FZ_REPS_TRIANGLE] == 1) {
		if (selItem == 0) {
			BKUser::options.txtFont = "bookr:builtin";
			buildOptionMenu();
			return BK_CMD_MARK_DIRTY;
		}
	}

	if (b[(BKUser::options.btnSwap?FZ_REPS_CIRCLE:FZ_REPS_CROSS)] == 1) {
		selItem = 0;
		topItem = 0;
		mode = BKMM_MAIN;
		BKUser::save();
		return BK_CMD_MARK_DIRTY;
	}

	if (b[FZ_REPS_START] == 1) {
		selItem = 0;
		topItem = 0;
		mode = BKMM_MAIN;
		BKUser::save();
		return BK_CMD_CLOSE_TOP_LAYER;
	}

	if (b[FZ_REPS_LEFT] == 1) {
		if (selItem == 1) {
			--BKUser::options.txtSize;
			if (BKUser::options.txtSize < 6) {
				BKUser::options.txtSize = 6;
			} else {
				buildOptionMenu();
			}
			return BK_CMD_MARK_DIRTY;
		} else if (selItem == 5) {
			--BKUser::options.pspSpeed;
			if (BKUser::options.pspSpeed < 0) {
				BKUser::options.pspSpeed = 0;
			} else {
				if (BKUser::options.pspSpeed == 0)
					FZScreen::setSpeed(5);
				else
					FZScreen::setSpeed(BKUser::options.pspSpeed);
				buildOptionMenu();
			}
			return BK_CMD_MARK_DIRTY;
		} else if (selItem == 8) {
			BKUser::options.btnSwap = BKUser::options.btnSwap ? 0 : 1;
			buildOptionMenu();
			return BK_CMD_MARK_DIRTY;
		}
	}

	if (b[FZ_REPS_RIGHT] == 1) {
		if (selItem == 1) {
			++BKUser::options.txtSize;
			if (BKUser::options.txtSize > 20) {
				BKUser::options.txtSize = 20;
			} else {
				buildOptionMenu();
			}
			return BK_CMD_MARK_DIRTY;
		} else if (selItem == 5) {
			++BKUser::options.pspSpeed;
			if (BKUser::options.pspSpeed > 6) {
				BKUser::options.pspSpeed = 6;
			} else {
				if (BKUser::options.pspSpeed == 0)
					FZScreen::setSpeed(5);
				else
					FZScreen::setSpeed(BKUser::options.pspSpeed);
				buildOptionMenu();
			}
			return BK_CMD_MARK_DIRTY;
		} else if (selItem == 8) {
			BKUser::options.btnSwap = BKUser::options.btnSwap ? 0 : 1;
			buildOptionMenu();
			return BK_CMD_MARK_DIRTY;
		}
	}

	return 0;
}
