/*
 * The Good Bookr - PSP Bible Reader
 * Copyright (C) 2007 Phosphorous http://www.pspcoder.net/goodbookr/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "bkosk.h"

string BKOsk::term = "";
vector<string> BKOsk::history;

BKOsk* BKOsk::create() {
	BKOsk* dz = new BKOsk();
	FZScreen::resetReps();
	term = "";
	if( history.empty() ){
		history.push_back("");
	} else {
		history.at(0) = "";
	}
	return dz;
}

int BKOsk::update(unsigned int buttons){

	sceCtrlPeekBufferPositive(&pad, 1);
	key = danzeff_readInput(pad);
	switch(key){
		case DANZEFF_NULL:
			break;
		// handle search history
		case DANZEFF_UP:
		case DANZEFF_DOWN:
			if( history.size() > 1 ){
				
				// save current input if first item
				if( historyItem == 0 ){
					history.at(0) = input;
				
				// clear history item if empty
				} else if( input.empty() ){
					historyIterator = history.begin();
					historyIterator += historyItem;
					history.erase(historyIterator);
					if( key == DANZEFF_DOWN ){ historyItem--; }
				}
				
				// cycle history item
				if( key == DANZEFF_UP ){
					if( historyItem == 0 ){
						historyItem = history.size()-1;
					} else {
						historyItem--;
					}
				} else if( key == DANZEFF_DOWN ){
					if( historyItem == history.size()-1 ){
						historyItem = 0;
					} else {
						historyItem++;
					}
				}
				
				// assign item and position cursor at text end
				input = history.at(historyItem);
				cursor = input.length();
				
			}
			break;
		case DANZEFF_LEFT:
			if( cursor > 0 ){ cursor--; }
			break;
		case DANZEFF_RIGHT:
			if( cursor < input.length() ){ cursor++; }
			break;
		case DANZEFF_START:
			term = input;
			if( !input.empty() ){
				// save input in history while preventing dupes and empties
				for( historyIterator = history.begin()+1; historyIterator < history.end(); historyIterator++ ){
					if( *historyIterator == input || *historyIterator == "" ){ history.erase(historyIterator); }
				}
				history.push_back(input);
			} else {
				// clear history item
				history.at(historyItem) = input;
			}
			input = "";
			return BK_CMD_SEARCH_RESULTS;
		case DANZEFF_SELECT:
			term = "";
			return BK_CMD_CLOSE_TOP_LAYER;
		case DANZEFF_DEL:
			if( !input.empty() && cursor != 0 ){
				input.erase( --cursor, 1 );
			}
			break;
		default:
			char c = (char)key;
			if( input.empty() || cursor == input.length() ){
				input.append( 1, c );
			} else {
				input.insert( cursor, 1, c );
			}
			cursor++;
			break;
	}

	return BK_CMD_MARK_DIRTY;

}

void BKOsk::render(){

	// draw input box
	drawTextBox(input, 250, 200, cursor, 0xf0222222, 0xffffffff, 0xff333333);

	// draw tip for accept/cancel
	texUI->bindForDisplay();
	FZScreen::ambientColor(0xff222222);
	drawPill(480 - 37 - 37 - 8, 272 - 18 - 4, 37*2 + 2, 50, 6, 31, 1);
	FZScreen::ambientColor(0xffffffff);
	// x, y, h, w, tx, ty
	drawImage(480 - 37 - 2, 272 - 18, 37, 18, 89, 108);
	drawImage(480 - 37 - 2 - 2 - 37, 272 - 18, 37, 18, 89, 87);

	// draw danzeff last since it seems to change color mode
	if( danzeff_isinitialized() ){
//		if(BKUser::options.txtRotation==1)
//			danzeff_set_orientation(DANZEFF_ORIENT_RIGHT);
//		else
//			danzeff_set_orientation(DANZEFF_ORIENT_UP);
		danzeff_render();
	}

}

void BKOsk::drawTextBox(string text, int w, int y, size_t cursorpos, int bg1, int bg2, int fg) {

	int x = (480 - w) / 2;

	texUI->bindForDisplay();

	// border
	FZScreen::ambientColor(bg1);
	drawPill(x, y, w, 30, 6, 31, 1);

	// box
	FZScreen::ambientColor(bg2);
	drawPill( x+5, y+5, w-10, 20, 6, 31, 1 );

	// text
	if( !text.empty() ){
		fontBig->bindForDisplay();
		FZScreen::ambientColor(fg);
		drawText( (char*)text.c_str(), fontBig, x+10, y+7 );
	}

	// cursor
	int txtw = 0;
	if( cursorpos != 0 ){
		text = text.substr( 0, cursorpos );
		txtw = textW( (char*)text.c_str(), fontBig );
		// adjust for leading spaces before cursor (textW bug?)
		size_t pos = 0;
		while( text[pos] == ' ' && pos < cursorpos ){ pos++; }
		if( pos == cursorpos ){ txtw += pos*4; }
	}
	texUI->bindForDisplay();
	FZScreen::ambientColor(fg|0x55);
	drawRect(x+10+txtw, y+10, 1, 16, 6, 31, 1);

}
