/*
 * The Good Bookr - PSP Bible Reader
 * Copyright (C) 2007 Phosphorous http://www.pspcoder.net/goodbookr/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef BKSQL_H
#define BKSQL_H

#include <sqlite3.h>
#include <string>
#include <vector>
using namespace std;

// constants for search menu
#define SEARCH_WITHIN_BOOK 100
#define SEARCH_WITHIN_OT 101
#define SEARCH_WITHIN_PENTATEUCH 105
#define SEARCH_WITHIN_OT_HISTORY 110
#define SEARCH_WITHIN_OT_POETRY 115
#define SEARCH_WITHIN_OT_MAJOR 120
#define SEARCH_WITHIN_OT_MINOR 125
#define SEARCH_WITHIN_NT 130
#define SEARCH_WITHIN_GOSPELS 135
#define SEARCH_WITHIN_EPISTLES 140
#define SEARCH_WITHIN_ALL 145

#define SEARCH_MATCH_PARTIAL 200
#define SEARCH_MATCH_FULL 205
#define SEARCH_MATCH_VERSE 210


class BKSql {

  private:

	sqlite3 *db;
	char **result;
	char *err;
	int rc;

	char tmpStr[1000]; // used w/ sprintf then appended to string

	string query;
	vector< vector<string> > bookAbbrevs;

	int init(sqlite3 *db);
	void splitString(string str, const char* delim, vector<string> *results){
		unsigned int pos;
		while( (pos = str.find_first_of(delim)) != string::npos ){
			if(pos!=0){
				results->push_back(str.substr(0,pos));
			}
			str = str.substr(pos+1);
		}
		if(str.length() && str.c_str() != delim) results->push_back(str);
	}

  public:

	struct BKBook{
		int bk;
		int ch;
		int vs;
		int tch;
		string name;
		BKBook() { }
		BKBook(string n, int b, int t) : bk(b), ch(1), vs(1), tch(t), name(n) { }
		BKBook(string n, int b, int c, int v) : bk(b), ch(c), vs(v), tch(0), name(n) { }
	};
// can this be a map instead?
	struct BKOption{
		int key;
		string label;
		BKOption(int k, string l) : key(k), label(l){ }
		BKOption(int k, const char* l) : key(k), label(l){ }
	};

	// constructor
	BKSql(string fileName = "./Bible"){
		// open db
		int rc = sqlite3_open(fileName.c_str(), &db);
		if(rc != SQLITE_OK){
			errorText = "Error loading Bible!\n";
			sqlite3_close(db);
		} else {
			init(db);
		}
	}
	// destructor
	~BKSql(){
		// close db
		sqlite3_close(db);
	}

	string loadBook(string* text, int bk=1, int ch=1, int vs=1);
	int searchBooks(string query, int book=0);

	static string errorText;

	static vector<BKBook> books;

	static string searchTerm;
	static unsigned int searchResult;
	static vector<BKBook> searchResults;

	// search options for search menu
	static unsigned int searchWithin;
	static vector<BKOption> searchWithins;
	static unsigned int searchMatch;
	static vector<BKOption> searchMatches;

};

#endif
